using System.Collections.Generic;

using skydiveLogs_api.Domain;


namespace skydiveLogs_api.Business.Interfaces
{
    public interface IStatsService
    {
        IEnumerable<Statistic> GetStatsByDz(User connectedUser);

        IEnumerable<Statistic> GetStatsByAircraft(User connectedUser);

        IEnumerable<Statistic> GetStatsByJumpType(User connectedUser);

        IEnumerable<Statistic> GetStatsByGear(User connectedUser);

        IEnumerable<Statistic> GetStatsByYear(User connectedUser);

        IEnumerable<Statistic> GetStatsForLastYearByDz(User connectedUser);

        IEnumerable<Statistic> GetStatsForLastYearByJumpType(User connectedUser);

        IEnumerable<Statistic> GetStatsForLastMonthByDz(User connectedUser);

        IEnumerable<Statistic> GetStatsForLastMonthByJumpType(User connectedUser);

        SimpleSummary GetSimpleSummary(User connectedUser);
    }
}
